///////////////////////////////////////////////////////////////////////////
// Expression compiler Ver. 2
///////////////////////////////////////////////////////////////////////////

#include <fstream>
#include <iostream>
#include <string>

// Intermediate representation
#include "AstManager.hh"

// Back End
#include "AsmGenerator.hh"

// Front End (via CppCC)
#include "expr_parser.hh"

using cppcc::ScanException;
using cppcc::ParseException;
using cppcc::ExprParser;


///////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[]) {
///////////////////////////////////////////////////////////////////////////
  
  if (argc < 1) {
    std::cerr << "No input file given" << std::endl;
  } else {
    std::ifstream *ifs = new std::ifstream(argv[1]);
    ExprParser parse(ifs);
    AstManager ast;
    AstNode* program;
    try {
      program = parse.program();
    } catch (ScanException &scex) {
      std::cerr << argv[1] << ": " << (std::string) scex << std::endl;
      return 0;
    } catch (ParseException &pex) {
      std::cerr << argv[0] << ": " << (std::string) pex << std::endl;
      return 0;
    } 
    AsmGenerator generateCode(std::cout);
    generateCode(program);
    ifs->close();
  }  
  return 0;

} // End of main
